import json
from urllib.request import Request, HTTPError, urlopen
from urllib.parse import urlencode
from http.client import HTTPResponse
from typing import Any, Literal

Method = Literal["GET", "POST"]

def http_request(
	url: str,
	error_hint: str,
	method: Method = "GET",
	headers: dict[str, str] = {},
	data: str | dict[str, Any] | None = None
) -> HTTPResponse:

	if isinstance(data, str):
		d = bytes(data, encoding="utf-8")
	elif isinstance(data, dict):
		if "Content-Type" in headers.keys() and headers["Content-Type"] == "application/x-www-form-urlencoded":
			d = bytes(urlencode(data), encoding="utf-8")
		else:
			d = json.dumps(data).encode("utf-8")
	else:
		d = None

	req = Request(url=url, data=d, headers=headers, method=method)

	try:
		return urlopen(req)
	except HTTPError as err:
		resp_text = err.read().decode()
		msg = f"{error_hint} problem, HTTP response code: {err.status}, response: {resp_text}"
		raise Exception(msg) from None



import logging
import threading
from typing import Any
from . import __version__ as icoscp_core_version

from .envri import EnvriConfig
from .http import http_request

def report_cpb_file_read(conf: EnvriConfig, hashes: list[str], columns: list[str] | None) -> None:
	access_info: dict[str, Any] = {
		"objects": hashes,
		"client": 'icoscp_core',
		"version": icoscp_core_version
	}
	if columns is not None:
		access_info["columns"] = columns

	payload = {"cpbFileAccess": access_info}

	return threading.Thread(target=_report_sync, args=(conf, payload)).start()

def _report_sync(conf: EnvriConfig, payload: dict[str, Any]) -> None:
	url = conf.auth_service_base_url + 'logs/portaluse'
	headers = {"Content-Type": "application/json"}
	try:
		http_request(url, f"Reporting data usage event to {url}", "POST", headers, payload)
	except Exception as err:
		logging.warning(str(err))
